@@ -22,6 +22,7 @@
 #include "diskcacheAPI/Cache/Directory.hh"
 #include "diskcacheAPI/Cache/DirectoryManager.hh"
 #include "diskcacheAPI/Cache/ExcludedDirectoriesSingleton.hh"
+#include "diskcacheAPI/Cache/ExcludedPattern.hh"
 #include "diskcacheAPI/Cache/QueryAnswer.hh"
 #include "diskcacheAPI/Cache/RegistrySingleton.hh"
 
@@ -44,6 +45,7 @@ using GenericAPI::LogEntryGroup_type;
 
 using diskCache::Cache::Devices;
 using diskCache::Cache::ExcludedDirectoriesSingleton;
+using diskCache::Cache::ExcludedPattern;
 
 typedef diskCache::Cache::Directory                     Directory;
 typedef diskCache::Cache::Directory::SymbolicLinkError  SymbolicLinkError;
@@ -56,6 +58,8 @@ typedef std::list< std::string >                        filename_container_type;
 typedef std::string                                     directory_element_type;
 typedef std::list< directory_element_type >             directory_container_type;
 
+#define VERBOSE_ERROR 0
+
 //-----------------------------------------------------------------------
 // Debugging
 //-----------------------------------------------------------------------
@@ -79,6 +83,15 @@ typedef std::list< directory_element_type >             directory_container_type
 #define	AT( a )	AT_BASE( a << std::endl << "\t" )
 #define	HERE( ) AT_BASE( "" )
 
+#if VERBOSE_ERROR
+#define COND_QUEUE_LOG_MESSAGE( MSG, MODE, LVL, CALLER, TYPE ) \
+    QUEUE_LOG_MESSAGE( MSG, MODE, LVL, CALLER, TYPE )
+#define COND_CALLER( A ) static const char* caller = A
+#else /* VERBOSE_ERROR */
+#define COND_QUEUE_LOG_MESSAGE( MSG, MODE, LVL, CALLER, TYPE )
+#define COND_CALLER( A )
+#endif /* VERBOSE_ERROR */
+
 //-----------------------------------------------------------------------
 // Forward declaration of local functions
 //-----------------------------------------------------------------------
@@ -269,6 +282,9 @@ namespace diskCache
         case Directory::DIRECTORY_REMOVED:
           state = "REMOVED";
           break;
+        case Directory::DIRECTORY_OFFLINE:
+          state = "OFFLINE";
+          break;
         }
         for ( ScanResults::file_container_type::const_iterator
                 fi_cur = cur->second.s_engines.begin( ),
@@ -441,14 +457,14 @@ namespace diskCache
           const std::string& Caller,
           const std::string& JobInfo ) const
     {
+      static const CHAR* const caller( "diskCache::Cache::Directory::Scan( ): " );
+
       dirref_type       retval;
       std::string       fullname( Fullname( ) );
 
       try
       {
 
-        static const CHAR* const caller( "diskCache::Cache::Directory::Scan( ): " );
-
         //---------------------------------------------------------------
         //
         //---------------------------------------------------------------
@@ -575,6 +591,12 @@ namespace diskCache
       }
       catch( const std::runtime_error& Except )
       {
+        COND_QUEUE_LOG_MESSAGE( "Exception: " << Except.what( )
+                                ,
+                                MT_DEBUG,
+                                50,
+                                caller,
+                                "CXX" );
         if ( errno == EINTR )
         {
           if ( device )
@@ -610,6 +632,30 @@ namespace diskCache
     read_directory( DirectoryScanData& ScanData,
                     const StatInfo& StatInfo )
     {
+      COND_CALLER( "Directory::read_directory" );
+      if ( m_state == DIRECTORY_OFFLINE )
+      {
+        if ( ! device )
+        {
+          device = diskCache::Cache::Devices::Find( Fullname( ) );
+        }
+        if ( ( device )
+             && ( device->IsOffline( ) == true ) )
+        {
+          //-------------------------------------------------------------
+          // The device is still offline so there is no use to do another
+          // stat which would just consume another thread.
+          //-------------------------------------------------------------
+          std::ostringstream	msg;
+
+          msg << "Device marked OFFLINE"
+            ;
+
+          errno = EINTR;
+          throw LDASTools::AL::TimeoutException( msg.str( ) );
+        }
+        m_state = DIRECTORY_NEW;
+      }
       //-----------------------------------------------------------------
       // Gather all the names in the directory skipping anything
       //     that matches the ignore pattern
@@ -617,6 +663,14 @@ namespace diskCache
       directory_container_type  cur_children;
       ignored_type              cur_ignored;
 
+      COND_QUEUE_LOG_MESSAGE( "Reading directory"
+                              << " Fullname: " << Fullname( )
+                              << " Root: " << Root( )
+                              ,
+                              MT_DEBUG,
+                              50,
+                              caller,
+                              "CXX" );
       try
       {
         ::read_directory( Fullname( ),
@@ -638,6 +692,7 @@ namespace diskCache
       {
         if ( errno == EINTR )
         {
+          m_state = DIRECTORY_OFFLINE;
           if ( device )
           {
             device->Offline( );
@@ -817,17 +872,20 @@ namespace
     }
     if( err != 0 )
     {
-      diskCache::Cache::Devices::element_type
-        dev = diskCache::Cache::Devices::Find( Path );
-      if ( dev )
+      if (  errno == EINTR )
       {
-        QUEUE_LOG_MESSAGE( "Flagging as OFFLINE: " << Path
-                           ,
-                           MT_DEBUG,
-                           50,
-                           method_name,
-                           "CXX" );
-        dev->Offline( );
+        diskCache::Cache::Devices::element_type
+          dev = diskCache::Cache::Devices::Find( Path );
+        if ( dev )
+        {
+          QUEUE_LOG_MESSAGE( "Flagging as OFFLINE: " << Path
+                             ,
+                             MT_NOTE,
+                             0,
+                             method_name,
+                             "CXX" );
+          dev->Offline( );
+        }
       }
       //-----------------------------------------------------------------
       // An error has occurred while trying to get information about
@@ -836,16 +894,16 @@ namespace
       //-----------------------------------------------------------------
       HERE( );
       string exc( method_name );
-      exc += "Could not execute lstat() for \'";
+      exc += "Could not get status for \'";
       exc += Path;
       exc += '\'';
       exc += errnoMessage( );
-      exc += " ";
+      exc += "\'";
       QUEUE_LOG_MESSAGE( "Exit: " << Path
                          << " Exception: " << exc
                          ,
-                         MT_DEBUG,
-                         50,
+                         MT_NOTE,
+                         0,
                          method_name,
                          "CXX" );
       throw std::runtime_error( exc );
@@ -1006,8 +1064,6 @@ namespace
     {
       try
       {
-        static const std::string tmp( ".tmp" );
-
         //-----------------------------------------------------------------
         // Within this loop, the file should be identified.
         // Subsequent checks will only be done if the previous check
@@ -1018,11 +1074,8 @@ namespace
         //-----------------------------------------------------------------
         if ( ( ( entry_cur->size( ) > 0 )
                && ( (*entry_cur)[ 0 ] == '.' ) )
-             || ( ( entry_cur->size( ) >= tmp.size( ) )
-                  && ( entry_cur->compare( entry_cur->size( ) -
-                                           tmp.size( ),
-                                           std::string::npos,
-                                           tmp ) == 0 ) ) )
+             || ( ExcludedPattern::IsExcluded( *entry_cur ) )
+             )
         {
             continue;
         }
