@@ -751,6 +751,15 @@ namespace diskCache
     void HotDirectory::
     HotLowerBound( timestamp_type Value )
     {
+      static const char* caller = "HotDirectory::HotLowerBound";
+      QUEUE_LOG_MESSAGE( "Changing hotdirectory lower bound from: " << m_lower_bound
+			 << " to: " << Value
+			 ,
+			 MT_NOTE,
+			 0,
+			 caller,
+			 "CXX" );
+      
       m_lower_bound = Value;
     }
 
@@ -814,6 +823,14 @@ namespace diskCache
     void HotDirectory::
     ScanInterval( timestamp_type Value )
     {
+      static const char* caller = "HotDirectory::ScanInterval";
+      QUEUE_LOG_MESSAGE( "Changing hotdirectory scan interval from: " << m_scan_interval
+			 << " to: " << Value
+			 ,
+			 MT_NOTE,
+			 0,
+			 caller,
+			 "CXX" );
       m_scan_interval = Value;
     }
 
