@@ -24,6 +24,8 @@ using LDASTools::AL::unordered_map;
 
 using GenericAPI::queueLogEntry;
 
+#define VERBOSE_ERROR 1
+
 #define SDGT_PATTERN "^([^\\-]+)\\-([^\\-]+)\\-([[:digit:]]+)\\-([[:digit:]]+)("
 #define SDGT_EXT_SUFFIX ")$"
 
@@ -124,6 +126,22 @@ namespace {
        time_type GapStart,
        time_type GapEnd )
   {
+#if VERBOSE_ERROR
+    static const char* caller = "anonymous::gap";
+#endif /* VERBOSE_ERROR */
+
+#if VERBOSE_ERROR
+    QUEUE_LOG_MESSAGE( "IntervalStart: " << IntervalStart
+		       << " IntervalEnd: " << IntervalEnd
+		       << " Duration: " << Duration
+		       << " GapStart: " << GapStart
+		       << " GapEnd: " << GapEnd
+		       ,
+		       MT_DEBUG,
+		       50,
+		       caller,
+		       "CXX" );
+#endif /* VERBOSE_ERROR */
     gap_compare_type	retval;
 
     time_type	offset;
@@ -170,7 +188,7 @@ namespace {
       GapEnd = IntervalEnd;
       retval = GAP_OVERLAP_TAIL;
     }
-    else if ( GapStart > IntervalEnd )
+    else if ( GapStart >= IntervalEnd )
     {
       retval = GAP_GREATER_THAN;
     }
@@ -181,8 +199,34 @@ namespace {
     }
     else
     {
+#if VERBOSE_ERROR
+      QUEUE_LOG_MESSAGE( "IntervalStart: " << IntervalStart
+			 << " IntervalEnd: " << IntervalEnd
+			 << " Duration: " << Duration
+			 << " GapStart: " << GapStart
+			 << " GapEnd: " << GapEnd
+			 << " gap: Unhandled pattern"
+			 ,
+			 MT_DEBUG,
+			 50,
+			 caller,
+			 "CXX" );
+#endif /* VERBOSE_ERROR */
       throw std::runtime_error( "gap: Unhandled pattern" );
     }
+#if VERBOSE_ERROR
+    QUEUE_LOG_MESSAGE( "IntervalStart: " << IntervalStart
+		       << " IntervalEnd: " << IntervalEnd
+		       << " Duration: " << Duration
+		       << " GapStart: " << GapStart
+		       << " GapEnd: " << GapEnd
+		       << " retval: " << retval
+		       ,
+		       MT_DEBUG,
+		       50,
+		       caller,
+		       "CXX" );
+#endif /* VERBOSE_ERROR */
     return retval;
   }
   //---------------------------------------------------------------------
@@ -752,6 +796,10 @@ namespace diskCache
     Find( const Directory& Dir,
 	  QueryAnswer& Answer ) const
     {
+#if VERBOSE_ERROR
+      static const char* caller = "SDGTx::DirectoryInfo::Find";
+#endif /* VERBOSE_ERROR */
+
       QueryAnswer::index_data_type
 	id( Answer.IndexData( ) );
 
@@ -867,38 +915,87 @@ namespace diskCache
 	      switch ( gap_class )
 	      {
 	      case GAP_GREATER_THAN:
+#if VERBOSE_ERROR
+		QUEUE_LOG_MESSAGE( "GAP_GREATER_THAN"
+				   ,
+				   MT_DEBUG, 50,
+				   caller,
+				   "CXX" );
+#endif /* VERBOSE_ERROR */
 		++interval_cur;
 		continue;
 		break;
 	      case GAP_LESS_THAN:
+#if VERBOSE_ERROR
+		QUEUE_LOG_MESSAGE( "GAP_LESS_THAN"
+				   ,
+				   MT_DEBUG, 50,
+				   caller,
+				   "CXX" );
+#endif /* VERBOSE_ERROR */
 		++gap_cur;
 		continue;
 		break;
 	      case GAP_OVERLAP_HEAD:
-		// Simple case where the end of the gap
-		//   needs to be adjusted since data has
-		//   been found
-		gap_cur->second = interval_cur->first;
-		gs = interval_cur->first;
-		++gap_cur;
+		{
+#if VERBOSE_ERROR
+		  QUEUE_LOG_MESSAGE( "GAP_OVERLAP_HEAD"
+				     ,
+				     MT_DEBUG, 50,
+				     caller,
+				     "CXX" );
+#endif /* VERBOSE_ERROR */
+		  time_type	dt = interval_cur->second.second;
+
+		  // Simple case where the end of the gap
+		  //   needs to be adjusted since data has
+		  //   been found
+		  gap_cur->second = interval_cur->first;
+		  gs = interval_cur->first;
+		  ge += ( ( dt - ( ge - interval_cur->first ) % dt ) % dt );
+		  ++gap_cur;
+		}
 		break;
 	      case GAP_OVERLAP_TAIL:
 		{
+#if VERBOSE_ERROR
+		  QUEUE_LOG_MESSAGE( "GAP_OVERLAP_TAIL"
+				     ,
+				     MT_DEBUG, 50,
+				     caller,
+				     "CXX" );
+#endif /* VERBOSE_ERROR */
 		  // Harder case. Need to delete the current
 		  // gap and create a new one.
 		  time_type	nge( gap_cur->second );
+		  time_type	dt = interval_cur->second.second;
 
 		  info->m_gaps.erase( gap_cur );
 		  info->m_gaps[ interval_cur->second.first ] = nge;
 		  nge = interval_cur->second.first;
+#if VERBOSE_ERROR
+		  QUEUE_LOG_MESSAGE( "nge: " << nge
+				     ,
+				     MT_DEBUG, 50,
+				     caller,
+				     "CXX" );
+#endif /* VERBOSE_ERROR */
 
 		  gap_cur = info->m_gaps.find( nge );
+		  gs -= ( gs - interval_cur->first ) % dt;
 		  ge = nge;
 		  ++interval_cur;
 		}
 		break;
 	      case GAP_MIDDLE:
 		{
+#if VERBOSE_ERROR
+		  QUEUE_LOG_MESSAGE( "GAP_MIDDLE"
+				     ,
+				     MT_DEBUG, 50,
+				     caller,
+				     "CXX" );
+#endif /* VERBOSE_ERROR */
 		  //-----------------------------------------------------
 		  // When the gap is in the middle of an interval,
 		  // then the gap is removed from the list of gaps
@@ -928,6 +1025,13 @@ namespace diskCache
 		break;
 	      case GAP_INTERVAL_MIDDLE:
 		{
+#if VERBOSE_ERROR
+		  QUEUE_LOG_MESSAGE( "GAP_INTERVAL_MIDDLE"
+				     ,
+				     MT_DEBUG, 50,
+				     caller,
+				     "CXX" );
+#endif /* VERBOSE_ERROR */
 		  time_type	nge( gap_cur->second );
 
 		  gap_cur->second = interval_cur->first;
@@ -942,6 +1046,18 @@ namespace diskCache
 	      //---------------------------------------------------------
 	      // Register the found files
 	      //---------------------------------------------------------
+#if VERBOSE_ERROR
+	      QUEUE_LOG_MESSAGE( "Register the found files:"
+				 << " ext_cur->first: " << ext_cur->first
+				 << " site_desc_cur->first: " << site_desc_cur->first
+				 << " gs: " << gs
+				 << " ge: " << ge
+				 << " duration: " << duration
+				 ,
+				 MT_DEBUG, 50,
+				 caller,
+				 "CXX" );
+#endif /* VERBOSE_ERROR */
 	      info->Fill( Dir.Fullname( ), // DirectoryName
 			  ext_cur->first,
 			  site_desc_cur->first,
@@ -1362,44 +1478,38 @@ namespace diskCache
     void SDGTx::QueryParams::
     Resample( )
     {
+      static const char* caller = "SDGTx::QueryParams::Resample";
+
       if ( m_gaps.size( ) == 0 )
       {
-	std::cerr << "DEBUG:"
-		  << " Gap size zero:"
-		  << " Resample: m_start: " << m_start
-		  << " m_stop: " << m_stop
-		  << std::endl
-	  ;
-	m_gaps[ m_start - 1 ] = m_start;
-	m_gaps[ m_stop ] = m_stop + 1;
+	QUEUE_LOG_MESSAGE( " Gap size zero:"
+			   << " Resample: m_start: " << m_start
+			   << " m_stop: " << m_stop
+			   ,
+			   MT_DEBUG, 40,
+			   caller,
+			   "SDGTx" );
+		  
+	//---------------------------------------------------------------
+	// Create a gap to get the frame file before
+	//---------------------------------------------------------------
+	m_gaps[ m_min_time - 1 ] = m_min_time;
+	//---------------------------------------------------------------
+	// Create a gap to get the frame file after
+	//---------------------------------------------------------------
+	m_gaps[ m_max_time ] = m_max_time + 1;
       }
       else
       {
-	std::cerr << "DEBUG:"
-		  << " File count: " << QueryAnswer::IndexDataBase::m_filenames.size( )
-		  << " Gap size NOT zero: (" << m_gaps.size( ) << ")"
-		  << " Resample: m_start: " << m_start
-		  << " m_stop: " << m_stop
-		  << std::endl
+	QUEUE_LOG_MESSAGE( " File count: " << QueryAnswer::IndexDataBase::m_filenames.size( )
+			   << " Gap size NOT zero: (" << m_gaps.size( ) << ")"
+			   << " Resample: m_start: " << m_start
+			   << " m_stop: " << m_stop
+			   ,
+			   MT_DEBUG, 40,
+			   caller,
+			   "SDGTx" );
 	  ;
-	{
-	  std::cerr << "DEBUG:Gaps remain at:"
-	    ;
-	  for ( gap_container_type::const_iterator
-		  cur = m_gaps.begin( ),
-		  last = m_gaps.end( );
-		cur != last;
-		++cur )
-	  {
-	    std::cerr << " [" << cur->first
-		      << ","
-		      << cur->second
-		      << ")"
-	      ;
-	  }
-	  std::cerr << std::endl
-	    ;
-	}
 	time_type	start, end;
 
 	//---------------------------------------------------------------
