@@ -20,6 +20,7 @@
 
 #include "diskcacheAPI/Common/Variables.hh"
 #include "diskcacheAPI/Cache/ExcludedDirectoriesSingleton.hh"
+#include "diskcacheAPI/Cache/ExcludedPattern.hh"
 #include "diskcacheAPI/Cache/HotDirectory.hh"
 
 #include "diskcacheAPI/IO.hh"
@@ -40,6 +41,8 @@ using diskCache::Commands::updateMountPtList;
 typedef diskCache::MetaCommand::client_type	client_type;
 typedef diskCache::MetaCommand::ClientServerInterface::ServerInfo ServerInfo;
 
+#define REQUEST_RELOAD LDASTools::AL::SignalHandler::SIGNAL_HANGUP
+
 namespace
 {
   CommandLineOptions
@@ -150,6 +153,11 @@ namespace diskCache
       {
 	state = BLOCK_EXCLUDED_DIRECTORIES;
       }
+      else if ( Value.compare( VAR_NAME_EXCLUDED_PATTERNS ) == 0 )
+      {
+	command.reset_excluded_patterns( );
+	state = BLOCK_EXCLUDED_PATTERNS;
+      }
       else if ( Value.compare( "MOUNT_POINTS" ) == 0 )
       {
 	state = BLOCK_MOUNT_POINTS;
@@ -252,6 +260,7 @@ namespace diskCache
 	}
 	break;
       case BLOCK_EXCLUDED_DIRECTORIES:
+      case BLOCK_EXCLUDED_PATTERNS:
       case BLOCK_EXTENSIONS:
       case BLOCK_MOUNT_POINTS:
 	//---------------------------------------------------------------
@@ -290,6 +299,9 @@ namespace diskCache
       case BLOCK_EXCLUDED_DIRECTORIES:
 	command.push_excluded_directory( Value );
 	break;
+      case BLOCK_EXCLUDED_PATTERNS:
+	command.push_excluded_pattern( Value );
+	break;
       case BLOCK_MOUNT_POINTS:
 	command.push_mount_point( Value );
 	break;
@@ -340,12 +352,42 @@ namespace diskCache
 			  "Comma seperated list of directories not to be searched",
 			  "list" ) );
 
+      retval.Add( Option( OPT_EXCLUDED_PATTERNS,
+			  "excluded-patterns",
+			  Option::ARG_REQUIRED,
+			  "Comma seperated list of file names not to be searched",
+			  "list" ) );
+
       retval.Add( Option( OPT_EXTENSIONS,
 			  "extensions",
 			  Option::ARG_REQUIRED,
 			  "Comma seperated list of file extensions",
 			  "list" ) );
 
+      retval.Add( Option( OPT_HOT_DIRECTORY_AGE,
+			  "hot-directory-age",
+			  Option::ARG_REQUIRED,
+			  "Age in seconds for which to consider a directory hot",
+			  "integer" ) );
+
+      retval.Add( Option( OPT_HOT_DIRECTORY_SCAN_INTERVAL,
+			  "hot-directory-scan-interval",
+			  Option::ARG_REQUIRED,
+			  "Number of seconds between scanning directories in hot list",
+			  "integer" ) );
+
+      retval.Add( Option( OPT_RWLOCK_INTERVAL,
+			  "rwlock-interval-ms",
+			  Option::ARG_REQUIRED,
+			  "milisecond interval",
+			  "integer" ) );
+
+      retval.Add( Option( OPT_RWLOCK_TIMEOUT,
+			  "rwlock-timeout-ms",
+			  Option::ARG_REQUIRED,
+			  "milisecond timeout",
+			  "integer" ) );
+
       retval.Add( Option( OPT_SCAN_INTERVAL,
 			  "interval",
 			  Option::ARG_REQUIRED,
@@ -390,6 +432,19 @@ namespace diskCache
       return retval;
     }
 
+    //-------------------------------------------------------------------
+    // Static data values of Daemon
+    //-------------------------------------------------------------------
+
+    const char* Daemon::VAR_NAME_CONCURRENCY = "CONCURRENCY";
+    const char* Daemon::VAR_NAME_EXCLUDED_PATTERNS = "EXCLUDED_PATTERN";
+    const char* Daemon::VAR_NAME_HOT_DIRECTORY_AGE =
+      "HOT_DIRECTORY_AGE_SEC";
+    const char* Daemon::VAR_NAME_HOT_DIRECTORY_SCAN_INTERVAL =
+      "HOT_DIRECTORY_SCAN_INTERVAL_SEC";
+    const char* Daemon::VAR_NAME_RWLOCK_INTERVAL = "RWLOCK_INTERVAL_MS";
+    const char* Daemon::VAR_NAME_RWLOCK_TIMEOUT = "RWLOCK_TIMEOUT_MS";
+
     //-------------------------------------------------------------------
     /// Contruct a new instance of Daemon.
     //-------------------------------------------------------------------
@@ -401,6 +456,7 @@ namespace diskCache
       : m_args( Args ),
 	cache_write_delay_( 120 ),
 	m_concurrency( 4 ),
+	excluded_patterns( Cache::ExcludedPattern::Get( ) ),
 	scan_interval( 16000 ),
 	version_binary( Streams::Interface::VERSION_NONE ),
 	finished( false ),
@@ -435,7 +491,7 @@ namespace diskCache
 	    parsing = false;
 	    break;
 	  case OPT_CONCURRENCY:
-	    diskCache::Common::Variables::Cache( "CONCURRENCY", arg_value );
+	    diskCache::Common::Variables::Cache( VAR_NAME_CONCURRENCY, arg_value );
 	    break;
 	  case OPT_CONFIGURATION_FILE:
 	    {
@@ -479,6 +535,29 @@ namespace diskCache
 	      Cache::ExcludedDirectoriesSingleton::Update( directories );
 	    }
 	    break;
+	  case OPT_EXCLUDED_PATTERNS:
+	    {
+	      //-----------------------------------------------------------
+	      // Generate list of directories
+	      //-----------------------------------------------------------
+	      size_t pos = 0;
+	      size_t end = 0;
+
+	      reset_excluded_patterns( );
+	      while ( end != std::string::npos )
+	      {
+		//-------------------------------------------------------
+		// Extract each directory
+		//-------------------------------------------------------
+		end = arg_value.find_first_of( ",", pos );
+		const size_t dend( ( end == std::string::npos )
+				   ? end
+				   : ( end - pos ) );
+		push_excluded_pattern( arg_value.substr( pos, dend ) );
+		pos = end + 1;
+	      }
+	    }
+	    break;
 	  case OPT_EXTENSIONS:
 	    {
 	      //-----------------------------------------------------------
@@ -497,6 +576,20 @@ namespace diskCache
 	      }
 	    }
 	    break;
+	  case OPT_HOT_DIRECTORY_AGE:
+	    diskCache::Common::Variables::Cache( VAR_NAME_HOT_DIRECTORY_AGE,
+						 arg_value );
+	    break;
+	  case OPT_HOT_DIRECTORY_SCAN_INTERVAL:
+	    diskCache::Common::Variables::Cache( VAR_NAME_HOT_DIRECTORY_SCAN_INTERVAL,
+						 arg_value );
+	    break;
+	  case OPT_RWLOCK_INTERVAL:
+	    diskCache::Common::Variables::Cache( VAR_NAME_RWLOCK_INTERVAL, arg_value );
+	    break;
+	  case OPT_RWLOCK_TIMEOUT:
+	    diskCache::Common::Variables::Cache( VAR_NAME_RWLOCK_TIMEOUT, arg_value );
+	    break;
 	  case OPT_SCAN_INTERVAL:
 	    set_scan_interval( arg_value );
 	    break;
@@ -524,9 +617,6 @@ namespace diskCache
 	  case OPT_OUTPUT_BINARY:
 	    set_output_binary( arg_value );
 	    break;
-	  case OPT_STAT_TIMEOUT:
-	    diskCache::Common::Variables::Cache( "STAT_TIMEOUT", arg_value );
-	    break;
 	  case OPT_VERSION_BINARY:
 	    set_output_binary_version( arg_value );
 	    break;
@@ -643,8 +733,8 @@ namespace diskCache
 			   0,
 			   method,
 			   "CXX" );
-	SignalHandler::Register( this, SignalHandler::SIGNAL_HANGUP );
-	SignalCapture( SignalHandler::SIGNAL_HANGUP );
+	SignalHandler::Register( this, REQUEST_RELOAD );
+	SignalCapture( REQUEST_RELOAD );
       }
       else
       {
@@ -653,8 +743,8 @@ namespace diskCache
 			   0,
 			   method,
 			   "CXX" );
-	SignalIgnore( SignalHandler::SIGNAL_HANGUP );
-	SignalHandler::Unregister( this, SignalHandler::SIGNAL_HANGUP );
+	SignalIgnore( REQUEST_RELOAD );
+	SignalHandler::Unregister( this, REQUEST_RELOAD );
       }
     }
 
@@ -662,22 +752,30 @@ namespace diskCache
     SignalCallback( signal_type Signal )
     {
       static const char* method = "Daemon::SignalCallback";
+
       QUEUE_LOG_MESSAGE( "Calling Daemon::SignalCallback"
 			 << " with signal: " << Signal,
 			 MT_NOTE,
 			 0,
 			 method,
 			 "CXX" );
-      if ( Signal == LDASTools::AL::SignalHandler::SIGNAL_HANGUP )
+      switch( Signal )
       {
+      case REQUEST_RELOAD:
 	//---------------------------------------------------------------
 	// Start the process of resetting
 	//---------------------------------------------------------------
-#if WORKING
-	reset( );
-#else
 	reset_requested = true;
-#endif /* WORKING */
+	break;
+      case LDASTools::AL::SignalHandler::SIGNAL_TERMINATE:
+	//---------------------------------------------------------------
+	// Start the process of resetting
+	//---------------------------------------------------------------
+	finished = true;
+	break;
+      default:
+	// Nothing
+	break;
       }
     }
 
@@ -743,8 +841,18 @@ namespace diskCache
     bool Daemon::
     process_cmd( CommandLineOptions& Options )
     {
+      static const char* caller = "diskCAche::MetaCommand::Daemon::process_cmd";
+
       bool	retval = true;
 
+      QUEUE_LOG_MESSAGE( "ENTRY:"
+			 << " Command: " << Options.ProgramName( )
+			 ,
+			 MT_DEBUG,
+			 10,
+			 caller,
+			 "CXX" );
+
       switch( CommandTable::Lookup( Options.ProgramName( ) ) )
       {
       case CommandTable::CMD_QUIT:
@@ -808,6 +916,21 @@ namespace diskCache
 	  cmd( );
 	}
 	break;
+      case CommandTable::CMD_RECONFIGURE:
+	{
+	  //-------------------------------------------------------------
+	  // Request the daemon to re-read configuration file
+	  //-------------------------------------------------------------
+	  MetaCommand::Reconfigure	cmd( Options, server_info );
+
+	  cmd.ClientHandle( client );
+	  cmd.File( configuration_filename );
+
+	  reset( );
+
+	  cmd( );
+	}
+	break;
       case CommandTable::CMD_STATUS:
 	{
 	  //-------------------------------------------------------------
@@ -822,10 +945,23 @@ namespace diskCache
 	break;
       default:
 	{
+	  QUEUE_LOG_MESSAGE( "Command unsupported: "
+			     << Options.ProgramName( )
+			     ,
+			     MT_NOTE,
+			     0,
+			     caller,
+			     "CXX" );
 	  retval = false;
 	}
 	break;
       }
+      QUEUE_LOG_MESSAGE( "EXIT:"
+			 ,
+			 MT_DEBUG,
+			 10,
+			 caller,
+			 "CXX" );
       return retval;
     }
 
@@ -835,6 +971,12 @@ namespace diskCache
       excluded_directories.push_back( Directory );
     }
     
+    inline void Daemon::
+    push_excluded_pattern( const std::string& Pattern )
+    {
+      excluded_patterns.push_back( Pattern );
+    }
+    
     inline void Daemon::
     push_extension( const std::string& Extension )
     {
@@ -863,6 +1005,12 @@ namespace diskCache
       return retval;
     }
 
+    inline void Daemon::
+    reset_excluded_patterns( )
+    {
+      excluded_patterns.resize( 0 );
+    }
+    
     inline void Daemon::
     cache_write_delay( const std::string& Value )
     {
@@ -941,11 +1089,22 @@ namespace diskCache
     void Daemon::
     operator()( )
     {
+      static const char* caller = "Daemon::operator()()";
       //-----------------------------------------------------------------
-      // 
+      // Setup to handle termination signal from the main thread
+      //   else we risk not being able to properly terminate the program.
       //-----------------------------------------------------------------
       daemon_mode = true;
 
+      QUEUE_LOG_MESSAGE( "Registering SIGNAL_TERMINATE handler",
+			 MT_NOTE,
+			 0,
+			 caller,
+			 "CXX" );
+      LDASTools::AL::SignalHandler::Register( this, LDASTools::AL::SignalHandler::SIGNAL_TERMINATE );
+      SignalCapture( LDASTools::AL::SignalHandler::SIGNAL_TERMINATE );
+
+      ResetOnSignal( true );
       ScanMountPointListContinuously( );
       DumpCacheDaemonStart( );
 
@@ -1034,6 +1193,13 @@ namespace diskCache
 	//---------------------------------------------------------------
 	Cache::ExcludedDirectoriesSingleton::Update( excluded_directories );
       }
+      if ( Mask & VAR_EXCLUDED_PATTERNS )
+      {
+	//---------------------------------------------------------------
+	// Update the list of directories to be excluded
+	//---------------------------------------------------------------
+	Cache::ExcludedPattern::Update( excluded_patterns );
+      }
       if ( ( Mask & VAR_EXTENSIONS )
 	   && ( m_extensions.size( ) > 0 ) )
       {
@@ -1083,7 +1249,38 @@ namespace diskCache
 	//---------------------------------------------------------------
 	// Setup scanning parameters
 	//---------------------------------------------------------------
-	diskCache::Common::Variables::Set( "CONCURRENCY" );
+	diskCache::Common::Variables::Set( VAR_NAME_CONCURRENCY );
+      }
+
+      if ( Mask & VAR_HOT_DIRECTORY_AGE )
+      {
+	//---------------------------------------------------------------
+	// Setup scanning parameters
+	//---------------------------------------------------------------
+	diskCache::Common::Variables::Set( VAR_NAME_HOT_DIRECTORY_AGE );
+      }
+
+      if ( Mask & VAR_HOT_DIRECTORY_SCAN_INTERVAL )
+      {
+	//---------------------------------------------------------------
+	// Setup scanning parameters
+	//---------------------------------------------------------------
+	diskCache::Common::Variables::Set( VAR_NAME_HOT_DIRECTORY_SCAN_INTERVAL );
+      }
+
+      if ( Mask & VAR_RWLOCK_INTERVAL )
+      {
+	//---------------------------------------------------------------
+	// Setup scanning parameters
+	//---------------------------------------------------------------
+	diskCache::Common::Variables::Set( VAR_NAME_RWLOCK_INTERVAL );
+      }
+
+      if ( Mask & VAR_RWLOCK_TIMEOUT )
+      {
+	//---------------------------------------------------------------
+	//---------------------------------------------------------------
+	diskCache::Common::Variables::Set( VAR_NAME_RWLOCK_TIMEOUT );
       }
 
       if ( Mask & VAR_SCAN_INTERVAL )
@@ -1153,6 +1350,11 @@ namespace diskCache
     reset( )
     {
       static const char* method_name = "diskCache::MetaCommand::Daemon::reset";
+      if ( reset_requested == false )
+      {
+	reset_requested = true;
+	return;
+      }
 
       reset_requested = false;
       //-----------------------------------------------------------------
